@@ -4,10 +4,19 @@ from requests_ntlm import HttpNtlmAuth
 class orchestrator(object):
     def __init__(self, host, user, password):
         self.host = host
-        self.user = user
-        self.password = password
         self.session = requests.session()
-        self.session.auth = HttpNtlmAuth(self.user,self.password)
+        self.session.auth = HttpNtlmAuth(user,password)
+
+    def intParseOutput(self, r):
+        output, output['result'] = {}, {}
+        doc = xmltodict.parse(r.text)
+        output['status'] = r.status_code
+        if r.status_code == 400: output['result']['message'] = doc['error']['message']['#text']
+        if r.status_code == 401: output['result']['message'] = "Authorization Required"
+        if r.status_code == 201 or r.status_code == 200:
+            return True, doc, output
+        else:
+            return False, doc, output        
         
     def Execute(self, runbook_id, params, dictionary=False):
         headers = {'Content-Type': 'application/atom+xml'}
@@ -16,29 +25,19 @@ class orchestrator(object):
             if data['status'] == 400: return data
         except: pass
         r = self.session.post(self.host + "/Jobs", data=data, headers=headers)
-        output, output['result'] = {}, {}
-        doc = xmltodict.parse(r.text)
-        output['status'] = r.status_code
-        if r.status_code == 400: 
-            try: output['result']['message'] = doc['error']['message']['#text']
-            except: pass
-        if r.status_code == 401: output['result']['message'] = "Authorization Required"
-        if r.status_code == 201:
-            properties = doc['entry']['content']['m:properties']
-            output['result']['id'] =  properties['d:Id']['#text']
-            output['result']['status'] = properties['d:Status']
-            output['result']['CreationTime'] = properties['d:CreationTime']['#text']
-            output['result']['LastModifiedTime'] = properties['d:LastModifiedTime']['#text']
+        success, doc, output = self.intParseOutput(r)
+        if not success: return output
+        properties = doc['entry']['content']['m:properties']
+        output['result']['id'] =  properties['d:Id']['#text']
+        output['result']['status'] = properties['d:Status']
+        output['result']['CreationTime'] = properties['d:CreationTime']['#text']
+        output['result']['LastModifiedTime'] = properties['d:LastModifiedTime']['#text']
         return output            
 
     def GetJobStatus(self, job_id):
         r = self.session.get(self.host + "/Jobs(guid'%s')" % job_id)
-        output, output['result'] = {}, {}
-        doc = xmltodict.parse(r.text)
-        output['status'] = r.status_code
-        if r.status_code == 401: output['result']['message'] = "Authorization Required"
-        if r.status_code == 400:
-            output['result']['message'] = doc['error']['message']['#text']
+        success, doc, output = self.intParseOutput(r)
+        if not success: return output
         properties = doc['entry']['content']['m:properties']
         output['result']['id'] =  properties['d:Id']['#text']
         output['result']['status'] = properties['d:Status']
@@ -46,15 +45,24 @@ class orchestrator(object):
         output['result']['LastModifiedTime'] = properties['d:LastModifiedTime']['#text']
         return output               
 
+    def GetJobInstance(self,job_id):
+        r = self.session.get(self.host + "/Jobs(guid'%s')/Instances" % job_id)
+        success, doc, output = self.intParseOutput(r)
+        if not success: return output
+        return doc['feed']['entry']['content']['m:properties']['d:Id']['#text']
+
+    def GetInstanceParameters(self, instance_id):
+        r = self.session.get(self.host + "/RunbookInstances(guid'%s')/Parameters" % instance_id)
+        success, doc, output = self.intParseOutput(r)
+        if not success: return output
+        for e in doc['feed']['entry']:
+            output['result'][str(e['content']['m:properties']['d:Name'])] = e['content']['m:properties']['d:Value']
+        return output
+
     def GetRunbooks(self):
         r = self.session.get(self.host + "/Runbooks")
-        output, output['result'] = {}, {}
-        doc = xmltodict.parse(r.text)
-        output['status'] = r.status_code
-        if r.status_code == 401: output['result']['message'] = "Authorization Required"
-        if r.status_code == 400:
-            output['result']['message'] = doc['error']['message']['#text']
-        if r.status_code != 200: return output
+        success, doc, output = self.intParseOutput(r)
+        if not success: return output
         for e in doc['feed']['entry']:
             output['result'][str(e['title']['#text'])] = e['content']['m:properties']['d:Id']['#text']
         return output
@@ -67,20 +75,12 @@ class orchestrator(object):
             output['status'] = 400
             output['message'] = "Runbook ID could not be found for %s" % runbook_name
             return output
-        output['status'] = 200
-        output['result']['id'] =  dict['result'][runbook_name]
-        output['result']['name'] = runbook_name
-        return output
+        return dict['result'][runbook_name]
 
     def GetParameters(self, runbook_id):
         r = self.session.get(self.host + "/Runbooks(guid'%s')/Parameters" % runbook_id)
-        output, output['result'] = {}, {}
-        doc = xmltodict.parse(r.text)
-        output['status'] = r.status_code
-        if r.status_code == 401: output['result']['message'] = "Authorization Required"
-        if r.status_code == 400: 
-            output['result']['message'] = doc['error']['message']['#text']
-        if r.status_code != 200: return output
+        success, doc, output = self.intParseOutput(r)
+        if not success: return output
         for e in doc['feed']['entry']:
             output['result'][str(e['title']['#text'])] = e['content']['m:properties']['d:Id']['#text']
         return output
